'''
main.py

This module acts as the entrypoint for the entire application

Attributes:
    app (QApplication): pyqt5 application instance

    window (MainWindow): window class invoked to render the ui
'''
import sys
import qdarkstyle
from PyQt5 import QtCore, QtGui, QtWidgets, QtWebEngineWidgets, uic
from ulanji.gui.main_window import MainWindow


def main():
    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()

    app.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())

    window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



'''libs/ulanji_content.py'''
from dataclasses import dataclass


@dataclass
class UlanjiContent:
    '''
    Content class to handle content operation

    Properties:
        content (str): string content
        char_count (int): total content character count
        file_name (str): path to currently opened file
        is_edited (bool): if the content is modified since last saved
        word_count (int): total content word count
    '''

    content: str = ''
    char_count: int = 0
    file_name: str = ''
    is_edited: bool = False
    word_count: int = 0

    def __init__(self):
        pass

    def calculate_word_count(self):
        '''
        Calculate word count from content
        '''
        self.word_count = len(self.content.split(' '))

    def calculate_char_count(self):
        '''
        Calculate character count from content
        '''
        self.char_count = self.content.count('')

    def set_content(self, file_path: str = ''):
        '''
        Set content from file
        '''
        try:
            with open(file_path, 'r') as file:
                self.file_name = file_path.split('/')[-1]
                self.file_path = file_path
                content = "".join(file.readlines())
                self.update_content(content)
        except IOError as error:
            print(error)

    def update_content(self, content: str = ''):
        '''
        Update content and recalculate counts
        '''
        self.content = content
        self.calculate_word_count()
        self.calculate_char_count()



'''main_window.py'''
from libs.content import UlanjiContent
from libs.renderer import UlanjiRenderer
from PyQt5 import uic
from PyQt5.QtWidgets import qApp, QAction, QFileDialog, QHBoxLayout, QLabel, QMainWindow, QSizePolicy, QSplitter
from PyQt5.QtWebChannel import QWebChannel
from PyQt5.QtCore import QUrl, QObject, pyqtSlot
import os

class MainWindow(QMainWindow):
    '''
    Main Window class inherited from QMainWindow

    Args:
        args (list): optional variable length argument list
        kwargs (list): optional keyword argument list
    '''

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ulanji = UlanjiContent()
        current_path = os.path.dirname(os.path.realpath(__file__))
        self.ui = uic.loadUi(f"{current_path}/main_window.ui", self)
        self.init_menubar_actions()
        self.init_menubar_shortcuts()
        self.init_statusbar()
        self.ui.editorPane.textChanged.connect(self.editor_pane_updated)
        self.channel = QWebChannel()
        self.renderer = UlanjiRenderer()
        self.channel.registerObject("backend", self.renderer)
        self.ui.rendererPane.page().setWebChannel(self.channel)
        file_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../assets/renderer.html"))
        local_url = QUrl.fromLocalFile(file_path)
        self.ui.rendererPane.load(local_url)

    def init_menubar_actions(self):
        '''
        Initialize menu bar action handler
        '''
        self.ui.menubar.triggered[QAction].connect(self.menubar_action_handler)

    def init_menubar_shortcuts(self):
        '''
        Initialize menu bar button shortcuts
        '''
        self.ui.actionNew.setShortcut("Ctrl+N")
        self.ui.actionOpen.setShortcut("Ctrl+O")
        self.ui.actionSave.setShortcut("Ctrl+S")
        self.ui.actionSave_As.setShortcut("Ctrl+Shift+S")
        self.ui.actionExit.setShortcut("Ctrl+Q")

    def init_statusbar(self):
        '''
        Initialize status bar widgets and components
        '''
        self.ui.statusbarFileNameLabel = QLabel('file:')
        self.ui.statusbarFileNameValue = QLabel('')
        self.statusBar().addPermanentWidget(self.ui.statusbarFileNameLabel)
        self.statusBar().addPermanentWidget(self.ui.statusbarFileNameValue)

        self.ui.statusbarWordCountLabel = QLabel('\twords:')
        self.ui.statusbarWordCountValue = QLabel('0')
        self.statusBar().addPermanentWidget(self.ui.statusbarWordCountLabel)
        self.statusBar().addPermanentWidget(self.ui.statusbarWordCountValue)

        self.ui.statusbarCharCountLabel = QLabel('\tchars:')
        self.ui.statusbarCharCountValue = QLabel('0')
        self.statusBar().addPermanentWidget(self.ui.statusbarCharCountLabel)
        self.statusBar().addPermanentWidget(self.ui.statusbarCharCountValue)

    def editor_pane_updated(self):
        self.ulanji.update_content(self.ui.editorPane.toPlainText())
        self.renderer.set_content(self.ui.editorPane.toPlainText())
        self.ui.rendererPane.reload()
        self.statusbar_update_values()

    def menubar_action_handler(self, action):
        '''
        Menu bar action handler
        '''
        try:
            getattr(
                self, f'menubar_action_{action.text().lower().strip("1&")}')()
        except AttributeError as error:
            print(error)

    def menubar_action_exit(self):
        '''
        Menu bar action: exit file
        '''
        # TODO: check if current content is edited
        qApp.quit()

    def menubar_action_new(self):
        '''
        Menu bar action: create new file
        '''
        # TODO: add check if current content is edited
        # TODO: add open new file

    def menubar_action_open(self):
        '''
        Menu bar action: open file
        '''
        # TODO: check if current content is edited and ask save
        options = QFileDialog.Options()
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Open file",
            "",
            "All Files (*)",
            options=options
        )

        if file_path:
            self.ulanji.set_content(file_path)
            self.ui.editorPane.setPlainText(self.ulanji.content)
            self.statusbar_update_values()

    def menubar_action_save(self):
        '''
        Menu bar action: save file
        '''
        # TODO: check if currrent content has file name

    def menubar_action_save_as(self):
        '''
        Menu bar action: save_as file
        '''
        # TODO: check if current content is edited

    def statusbar_update_values(self):
        '''
        Update status bar values on change content
        '''
        self.ui.statusbarFileNameValue.setText(self.ulanji.file_name)
        self.ui.statusbarWordCountValue.setText(str(self.ulanji.word_count))
        self.ui.statusbarCharCountValue.setText(str(self.ulanji.char_count))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "ulanji",
    version = "0.0.6",
    author = "Jerven Clark Chua",
    author_email = "jervenclark@gmail.com",
    description = "Distraction free markdown editor",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/jervenclark/ulanji",
    packages = setuptools.find_packages(),
    package_data = {
        'ulanji.gui': ['*.ui'],
        'ulanji.assets': ['*.html']
    },
    include_package_data = True,
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires = '>=3.6',
    install_requires = [
        'markdown2==2.3.9',
        'PyQt5==5.15.0',
        'PyQtWebEngine==5.15.0',
        'QDarkStyle==2.8.1',
    ]
)



from markdown2 import markdown
from PyQt5.QtCore import QObject, pyqtSlot


class UlanjiRenderer(QObject):

    content = ''

    def __init__(self):
        super().__init__()
        self.markdown_extras = [
            'code-friendly',
            'code-color',
            'fenced-code-blocks',
            'footnotes',
            'header-ids',
            'tables'
        ]

    @pyqtSlot(result=str)
    def getStr(self):
        return self.content

    @pyqtSlot(str)
    def printStr(self, content):
        print(content)

    def set_content(self, content):
        self.content = markdown(content, extras=self.markdown_extras)
